@@ -18,14 +18,11 @@
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <gconf/gconf-client.h>
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 
 #include "gsd-ldsm-trash-empty.h"
 
-#define NAUTILUS_CONFIRM_TRASH_KEY "/apps/nautilus/preferences/confirm_trash"
-
 /* Some of this code has been borrowed from the trash-applet, courtesy of Ryan Lortie */
 
 static GtkWidget *trash_empty_confirm_dialog = NULL;
@@ -318,27 +315,27 @@ trash_empty_confirmation_response (GtkDialog *dialog,
         if (response_id == GTK_RESPONSE_YES)
                 trash_empty_start ();
 
-        gtk_object_destroy (GTK_OBJECT (dialog));
+        gtk_widget_destroy (GTK_WIDGET (dialog));
         trash_empty_confirm_dialog = NULL;
 }
 
 static gboolean
-trash_empty_require_confirmation ()
+trash_empty_require_confirmation (void)
 {
-        GConfClient *client;
         gboolean require_confirmation = TRUE;
-        GError *error = NULL;
+        const char * const *schemas;
+        guint i;
 
-        client = gconf_client_get_default ();
-        if (client) {
-                require_confirmation = gconf_client_get_bool (client, NAUTILUS_CONFIRM_TRASH_KEY, &error);
-                if (error) {
-                        g_warning ("Failed to read confirm_trash key from GConf: %s", error->message ? error->message : "Unknown error");
-                        /* It's safest to assume that confirmation is required here */
-                        require_confirmation = TRUE;
-                        g_error_free (error);
+        schemas = g_settings_list_schemas ();
+        for (i = 0; schemas[i] != NULL; i++) {
+                if (g_str_equal (schemas[i], "org.gnome.nautilus.preferences")) {
+                        GSettings *settings;
+
+                        settings = g_settings_new ("org.gnome.nautilus.preferences");
+                        require_confirmation = g_settings_get_boolean (settings, "confirm-trash");
+                        g_object_unref (settings);
+                        break;
                 }
-                g_object_unref (client);
         }
 
         return require_confirmation;
@@ -387,7 +384,7 @@ trash_empty_show_confirmation_dialog ()
 }
 
 void
-gsd_ldsm_trash_empty ()
+gsd_ldsm_trash_empty (void)
 {
         if (trash_empty_confirm_dialog)
                 gtk_window_present (GTK_WINDOW (trash_empty_confirm_dialog));
